"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WifiConfProtocolBase = void 0;
const qcloud_iotexplorer_appdev_sdk_1 = require("qcloud-iotexplorer-appdev-sdk");
const constants_1 = require("../constants");
const utils_1 = require("../utils");
const issueHandler_1 = require("./issueHandler");
const queryTokenStateAndBind_1 = require("./queryTokenStateAndBind");
/**
 * 配网过程抽象为：配网协议 - 自定义业务流程 - 结束
 * （适用于非softap协议）
 */
class WifiConfProtocolBase extends qcloud_iotexplorer_appdev_sdk_1.AppDevSdk.utils.EventEmitter {
    constructor(sdk, options) {
        super();
        // 必须设置当前协议类型，否则无法正确上报，可选值：'SmartConfig' | 'AirKiss' | 'SimpleConfig'
        this.connectAborted = false;
        this.options = {
            udpPort: 8266,
            stepInterval: 1000,
            familyId: 'default',
            autoRetry: false,
            autoBind: true,
        };
        this.sdk = sdk;
        Object.assign(this.options, options);
    }
    onProgress(event) {
        if (typeof this.options.onProgress === 'function') {
            this.options.onProgress(event);
        }
    }
    onComplete(data) {
        if (typeof this.options.onComplete === 'function') {
            this.options.onComplete(data);
        }
    }
    onError(event) {
        if (typeof this.options.onError === 'function') {
            this.options.onError(event);
        }
    }
    async start({ doNotRetry = false } = {}) {
        var _a, _b;
        this.onProgress({ code: "WIFI_CONF_START" /* WifiConfStepCode.WIFI_CONF_START */ });
        const { targetWifiInfo: { SSID, BSSID, password }, } = this.options;
        // 上报目前的ssid和密码
        this.onProgress({
            code: "PROTOCOL_START" /* WifiConfStepCode.PROTOCOL_START */,
            detail: {
                SSID,
                password,
            },
            reportEvents: ['配网', '发送wifi'],
        });
        try {
            const startTime = Date.now();
            let address;
            try {
                const resp = await this.doProtocol({
                    SSID,
                    BSSID,
                    password,
                });
                address = resp ? resp.address : null;
            }
            catch (err) {
                console.error(err);
                throw Object.assign({ code: 'PROTOCOL_FAIL' }, err);
            }
            if (address) {
                // TODO: 小程序处主动监听这个事件，额外进行性能上报
                // TODO: 需要上报配网类型？
                this.onProgress({
                    code: "PROTOCOL_SUCCESS" /* WifiConfStepCode.PROTOCOL_SUCCESS */,
                    detail: {
                        data: { address },
                        timeCost: Date.now() - startTime,
                    },
                    reportEvents: ['发送wifi'],
                });
            }
            else {
                throw {
                    code: 'PROTOCOL_INVALID_RESPONSE',
                };
            }
            const deviceInfo = await this.doBusiness({ address });
            // TODO: 小程序处主动监听这个事件，额外进行性能上报
            this.onProgress({
                code: "WIFI_CONF_SUCCESS" /* WifiConfStepCode.WIFI_CONF_SUCCESS */,
                detail: Object.assign({ timeCost: Date.now() - startTime }, deviceInfo),
                reportEvents: [
                    '配网',
                ],
            });
            this.onComplete(deviceInfo);
        }
        catch (error) {
            console.error(error);
            if (error && error.code in constants_1.WifiConfErrorMsg) {
                error.uiMsg = constants_1.WifiConfErrorMsg[error.code];
            }
            if (((_a = error === null || error === void 0 ? void 0 : error.detail) === null || _a === void 0 ? void 0 : _a.code) === "QUERY_TOKEN_RESULT_ERROR" /* WifiConfStepCode.QUERY_TOKEN_RESULT_ERROR */) {
                error.uiMsg = (_b = error === null || error === void 0 ? void 0 : error.detail) === null || _b === void 0 ? void 0 : _b.msg;
            }
            if (!doNotRetry && this.options.autoRetry) {
                const isHandled = await (0, issueHandler_1.issueHandler)({
                    error,
                    targetWifiInfo: this.options.targetWifiInfo,
                    wifiConfType: this.options.wifiConfType,
                    onProgress: (data) => {
                        this.onProgress.call(this, data);
                    },
                });
                console.log(`handle udp error, isHandled: ${isHandled}`);
                if (isHandled) {
                    this.onProgress({ code: "WIFI_CONF_AUTO_RETRY" /* WifiConfStepCode.WIFI_CONF_AUTO_RETRY */ });
                    return this.start({ doNotRetry: true });
                }
            }
            this.connectAborted = true;
            // this.onProgress({
            //   code: WifiConfStepCode.WIFI_CONF_FAIL,
            //   detail: { error },
            // });
            // add event,to tam count
            this.onProgress({
                code: "WIFI_CONF_FAIL" /* WifiConfStepCode.WIFI_CONF_FAIL */,
                detail: { error },
            });
            this.onError({ code: 'WIFI_CONF_FAIL', detail: { error } });
        }
    }
    // 需要返回 { address }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    doProtocol(any) {
        throw new Error('子类必须自行实现自己协议传输逻辑');
    }
    async doBusiness({ address, }) {
        this.onProgress({ code: "BUSINESS_START" /* WifiConfStepCode.BUSINESS_START */ });
        this.onProgress({ code: "CREATE_UDP_CONNECTION_START" /* WifiConfStepCode.CREATE_UDP_CONNECTION_START */, reportEvents: ['udp'] });
        const startSendToken = Date.now();
        const udpServer = new utils_1.UdpServer({
            address,
            port: this.options.udpPort,
            sdk: this.sdk,
        });
        // 1. 建立连接
        const onErrorPromise = qcloud_iotexplorer_appdev_sdk_1.AppDevSdk.utils.genPromise();
        const onProgressErrorPromise = qcloud_iotexplorer_appdev_sdk_1.AppDevSdk.utils.genPromise();
        udpServer.onError(errMsg => onErrorPromise.reject({
            code: 'UDP_ERROR',
            errMsg,
        }));
        udpServer.onProgressError(message => onProgressErrorPromise.reject({
            code: 'BUSINESS_DEVICE_ERROR',
            message,
        }));
        const doConnect = async () => {
            try {
                const stepCheck = async (duration) => {
                    await qcloud_iotexplorer_appdev_sdk_1.AppDevSdk.utils.delay(duration || this.options.stepInterval);
                    if (this.connectAborted) {
                        qcloud_iotexplorer_appdev_sdk_1.AppDevSdk.utils.logger.debug('connection aborted');
                    }
                };
                await stepCheck();
                // 除了softap其他都只有2.0
                const protoVersion = '2.0';
                let deviceExtendInfo = {};
                console.log('getDeviceExtendInfo', this.options.getDeviceExtendInfo);
                if (typeof this.options.getDeviceExtendInfo === 'function') {
                    deviceExtendInfo = await this.options.getDeviceExtendInfo({
                        udpServer,
                    });
                }
                // 2. 发送目标wifi信息
                const response = await udpServer.send(Object.assign({ cmdType: 0, token: this.options.wifiConfToken }, deviceExtendInfo), {
                    // 验证回包
                    validateResponse(message) {
                        console.log('cmdType', message.cmdType);
                        if (+message.cmdType === 2) {
                            return true;
                        }
                        return false;
                    },
                });
                // TODO: 上报
                this.onProgress({
                    code: "CREATE_UDP_CONNECTION_SUCCESS" /* WifiConfStepCode.CREATE_UDP_CONNECTION_SUCCESS */,
                    detail: {
                        data: response,
                        timeCost: Date.now() - startSendToken,
                        protoVersion,
                    },
                    reportEvents: ['udp'],
                });
                udpServer.destroy();
                await stepCheck();
                const { productId, deviceName } = response;
                await (0, queryTokenStateAndBind_1.queryTokenStateAndBind)({
                    token: this.options.wifiConfToken,
                    productId,
                    deviceName,
                    familyId: this.options.familyId,
                    roomId: this.options.roomId,
                    onProgress: (data) => {
                        this.onProgress.call(this, data);
                    },
                    sdk: this.sdk,
                    newTokenVersion: this.options.newTokenVersion,
                });
                return { productId, deviceName };
            }
            catch (err) {
                udpServer.destroy();
                throw err;
            }
        };
        const response = await Promise.race([
            doConnect(),
            onErrorPromise.promise,
            onProgressErrorPromise.promise,
        ]);
        // this.onProgress({
        //   code: WifiConfStepCode.BUSINESS_SUCCESS,
        //   detail: {
        //     response,
        //   },
        // });
        // add event,to tam count
        this.onProgress({
            code: "BUSINESS_SUCCESS" /* WifiConfStepCode.BUSINESS_SUCCESS */,
            detail: {
                response,
            },
            reportEvents: [
                'addDevice',
            ],
        });
        return response;
    }
}
exports.WifiConfProtocolBase = WifiConfProtocolBase;
//# sourceMappingURL=WifiConfProtocolBase.js.map