"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UdpServer = void 0;
const qcloud_iotexplorer_appdev_sdk_1 = require("qcloud-iotexplorer-appdev-sdk");
const decodeUdpMsg = (message) => {
    const unit8Arr = new Uint8Array(message);
    const encodedString = String.fromCharCode.apply(null, unit8Arr);
    return decodeURIComponent(escape(encodedString)); // 没有这一步中文会乱码
};
// TODO 重构，抽离业务逻辑
class UdpServer {
    constructor({ address, port, sdk, retryTime = 5, retryGap = 2000, failRetryGap = 200, messageNoParse = false, retryWhenFail = false, }) {
        Object.assign(this, {
            address,
            port,
            sdk,
            retryTime,
            retryGap,
            // 不要json.parse的处理，日志模式不是json方式
            messageNoParse,
            retryWhenFail,
            failRetryGap,
        });
        // eslint-disable-next-line no-multi-assign
        const socket = (this.socket = wx.createUDPSocket());
        socket.bind(this.port);
        const onError = error => this._onErrorHandler(error);
        const onMessage = resp => this._onMessageHandler(resp);
        socket.onError(onError);
        socket.onMessage(onMessage);
        this.destroy = () => {
            this.socket.offError(onError);
            this.socket.offMessage(onMessage);
            this.socket.close();
        };
    }
    _onErrorHandler(errMsg) {
        if (typeof this._errorHandler === 'function') {
            this._errorHandler(errMsg);
        }
        else {
            console.warn('UdpServer unhandled error', errMsg);
        }
    }
    _onMessageHandler(resp) {
        var _a, _b, _c;
        try {
            // 如果不需要json.parse
            if (this.messageNoParse) {
                const message = decodeUdpMsg(resp.message);
                (_a = this.sdk) === null || _a === void 0 ? void 0 : _a.reporter.info('udp-on-message', {
                    data: { message },
                });
                if (typeof this._msgHandler === 'function') {
                    this._msgHandler(message);
                }
                else {
                    console.warn('UdpServer unhandled msg', message);
                }
                return;
            }
            const message = JSON.parse(decodeUdpMsg(resp.message));
            (_b = this.sdk) === null || _b === void 0 ? void 0 : _b.reporter.info('udp-on-message', {
                data: { message },
            });
            this._cmdTypeHandler(message);
        }
        catch (error) {
            (_c = this.sdk) === null || _c === void 0 ? void 0 : _c.reporter.info('udp-parse-message-error', {
                error,
            });
        }
    }
    _cmdTypeHandler(message) {
        var _a;
        const callMessageHandler = (message) => {
            // 每个步骤自行注册回调，来判断自己的步骤是否有收到期望的回包
            // eslint-disable-next-line no-lonely-if
            if (typeof this._msgHandler === 'function') {
                this._msgHandler(message);
            }
            else {
                console.warn('UdpServer unhandled msg', message);
            }
        };
        // 模组回包有cmdType为2和5的情况
        console.log('------', message);
        switch (+message.cmdType) {
            case 2:
                // 模组回异常，直接中断过程报错
                if (message.deviceReply === 'Current_Error') {
                    this._onProgressErrorHandler({
                        code: 'BUSINESS_DEVICE_ERROR',
                        detail: message,
                    });
                }
                else if (message.deviceReply === 'Previous_Error') {
                    // 上一次连接过程中发生的，还未来得及发出去的错误信息，直接上报，不产生副作用
                    (_a = this.sdk) === null || _a === void 0 ? void 0 : _a.reporter.info('softap-receive-prev-error', {
                        data: { message },
                    });
                }
                else {
                    callMessageHandler(message);
                }
                break;
            case 5:
            case 7:
                callMessageHandler(message);
                break;
        }
    }
    _onProgressErrorHandler(error) {
        if (typeof this._progressErrorHandler === 'function') {
            this._progressErrorHandler(error);
        }
        else {
            console.warn('UdpServer unhandled progress error', error);
        }
    }
    onError(fn) {
        this._errorHandler = fn;
        return this;
    }
    onProgressError(fn) {
        this._progressErrorHandler = fn;
        return this;
    }
    onMessage(fn) {
        this._msgHandler = fn;
        return this;
    }
    _send(message) {
        if (typeof message !== 'string') {
            message = JSON.stringify(message);
        }
        this.socket.send({
            address: this.address,
            port: this.port,
            message,
        });
    }
    send(msg, { validateResponse } = {}) {
        this.aborted = false;
        return new Promise(async (resolve, reject) => {
            try {
                let promisePending = true;
                let retryCount = 0;
                let failRetryCount = 0;
                this.onMessage((message) => {
                    try {
                        promisePending = false;
                        // 验证回包格式
                        if (!validateResponse
                            || (typeof validateResponse === 'function'
                                && validateResponse(message) === true)) {
                            resolve(message);
                        }
                    }
                    catch (err) {
                        reject(err);
                    }
                });
                const doSend = () => {
                    var _a, _b;
                    (_b = (_a = this.sdk) === null || _a === void 0 ? void 0 : _a.reporter) === null || _b === void 0 ? void 0 : _b.info('udp-send-msg', {
                        data: {
                            msg,
                            retryCount,
                            failRetryCount,
                        },
                    });
                    this._send(msg);
                };
                try {
                    doSend();
                }
                catch (err) {
                    // udp_error 错误情况是否重试处理
                    console.log('come handle udp error', failRetryCount);
                    if (this.retryWhenFail) {
                        const doFailRetry = async () => {
                            console.log('retry send udp');
                            try {
                                if (failRetryCount < this.retryTime) {
                                    await qcloud_iotexplorer_appdev_sdk_1.AppDevSdk.utils.delay(this.failRetryGap);
                                    failRetryCount++;
                                    doSend();
                                }
                                else {
                                    reject(Object.assign({ code: 'UDP_SEND_MSG_FAIL' }, err));
                                }
                            }
                            catch (err) {
                                await doFailRetry();
                            }
                        };
                        await doFailRetry();
                    }
                    else {
                        reject(Object.assign({ code: 'UDP_SEND_MSG_FAIL' }, err));
                    }
                }
                while (!this.aborted
                    && promisePending
                    && retryCount <= this.retryTime) {
                    await qcloud_iotexplorer_appdev_sdk_1.AppDevSdk.utils.delay(this.retryGap);
                    if (this.aborted) {
                        return reject(null);
                    }
                    if (promisePending) {
                        retryCount++;
                        doSend();
                    }
                    else {
                        return;
                    }
                }
                if (this.aborted) {
                    return reject(null);
                }
                reject({ code: 'UDP_NOT_RESPONSE' });
            }
            catch (err) {
                reject(Object.assign({ code: 'UDP_SEND_MSG_FAIL' }, err));
            }
        });
    }
    abort() {
        this.aborted = true;
    }
}
exports.UdpServer = UdpServer;
//# sourceMappingURL=UdpServer.js.map