/**
 * @description 自动处理一些配网过程的错误，这样降低用户的操作成本，优化体验
 * @description type1: 当当前的wifi被系统切走的时候，尝试帮用户切回来，然后再重试配网
 */
import { AppDevSdk } from 'qcloud-iotexplorer-appdev-sdk';
import wxApis from '../utils/wxApis';
import { checkIsIOS } from '../utils';
const { delay } = AppDevSdk.utils;
const UDP_ERROR_WAIT_RETRY_TIME = 1500;
const WIFI_CONNECT_WAIT_RETRY_TIME = 4000;
const handleWifiIssue = async ({ 
// code,
error, targetWifiInfo, onProgress, }) => {
    // targetWifiInfo = {
    // 	SSID: 'L-0558',
    // 	password: 'video123.'
    // }
    onProgress({
        code: "AUTO_HANDLE_WIFI_PROBLEM_START" /* WifiConfStepCode.AUTO_HANDLE_WIFI_PROBLEM_START */,
        detail: {
            targetWifiInfo,
        },
    });
    // 检查是否wifi连接错误
    try {
        // 根据错误码数增加对本地网络权限开关未打开的检查
        const { code = ' ', errMsg = '' } = error;
        const isIOS = checkIsIOS();
        if (isIOS) {
            const testNoRouteReg = new RegExp(/no route to host/i);
            if (testNoRouteReg.test(errMsg)) {
                const isConfirm = await AppDevSdk.utils.tips.showModal({
                    title: '需要先开启微信本地网络权限才能继续配网',
                    confirmText: '去开启',
                    confirmColor: AppDevSdk.constants.themeColorMap.primary,
                    cancelText: '取消',
                    cancelColor: AppDevSdk.constants.themeColorMap.danger,
                });
                if (!isConfirm) {
                    return false;
                }
                else {
                    try {
                        await wxApis.getWifiList();
                        await new Promise((resolve) => {
                            wxApis.onAppShow(() => {
                                // 小程序无法获取用户是否开启了“本地网络”，用户返回连连后，默认用户已经开启了
                                resolve(true);
                            });
                        });
                        onProgress({
                            code: "AUTO_HANDLE_WIFI_NO_ROUTE" /* WifiConfStepCode.AUTO_HANDLE_WIFI_NO_ROUTE */,
                            detail: {
                                msg: '处理用户本地网络设置开关没打开的情况',
                            },
                        });
                        return true;
                    }
                    catch (error) {
                        console.log('handle no route error fail', error);
                        return false;
                    }
                }
            }
        }
        const { wifi: connectedWifi } = await wxApis.getConnectedWifi();
        // wifi匹配，说明是其他的问题导致的报错
        if (connectedWifi && connectedWifi.SSID === targetWifiInfo.SSID) {
            // 如果code是UDP相关的，可能是wifi切换过来之后，底层udp还不能切换过来，会发包失败
            if (code === 'UDP_SEND_MSG_FAIL' || code === 'UDP_ERROR') {
                await delay(UDP_ERROR_WAIT_RETRY_TIME);
                onProgress({
                    code: "AUTO_HANDLE_WIFI_PROBLEM_SUCCESS" /* WifiConfStepCode.AUTO_HANDLE_WIFI_PROBLEM_SUCCESS */,
                    detail: {
                        msg: 'UDP错误，或许是连接wifi的延迟，直接重试下',
                    },
                });
                return true;
            }
            onProgress({
                code: "AUTO_HANDLE_WIFI_PROBLEM_CANCEL" /* WifiConfStepCode.AUTO_HANDLE_WIFI_PROBLEM_CANCEL */,
                detail: {
                    msg: '当前连接的wifi正确，不需要处理',
                },
            });
            return false;
        }
    }
    catch (error) {
        console.error('getConnectedWifi failed', error);
        if (error.errCode && error.errCode !== 12010) {
            // 如果没有得到wifi信息就算了,因为可能wifi模块没有开启
            onProgress({
                code: "AUTO_HANDLE_WIFI_PROBLEM_CANCEL" /* WifiConfStepCode.AUTO_HANDLE_WIFI_PROBLEM_CANCEL */,
                detail: Object.assign({ msg: '获取当前连接的wifi出错 ' }, error),
            });
            return false;
        }
        // 无网络连接的情况下，让重新连接下，其余的错误是wifi模块的问题，就不做处理了
    }
    try {
        // 防止没有启动
        await wxApis.startWifi();
        await wxApis.connectWifi({
            SSID: targetWifiInfo.SSID,
            password: targetWifiInfo.password,
        });
        await delay(WIFI_CONNECT_WAIT_RETRY_TIME);
        onProgress({
            code: "AUTO_HANDLE_WIFI_PROBLEM_SUCCESS" /* WifiConfStepCode.AUTO_HANDLE_WIFI_PROBLEM_SUCCESS */,
            detail: {
                handled: true,
            },
        });
        return true;
    }
    catch (error) {
        console.error(error);
        AppDevSdk.utils.logger.debug('connectWifi fail', error);
        // 连接wifi出错之后，提醒下用户吧
        const isConfirm = await AppDevSdk.utils.tips.showModal({
            title: `手机连接${targetWifiInfo.SSID}失败，请在设置页面手动切换到${targetWifiInfo.SSID}再继续`,
            confirmText: '继续',
            confirmColor: AppDevSdk.constants.themeColorMap.primary,
            cancelText: '取消',
            cancelColor: AppDevSdk.constants.themeColorMap.danger,
        });
        if (!isConfirm) {
            onProgress({
                code: "AUTO_HANDLE_WIFI_PROBLEM_CANCEL" /* WifiConfStepCode.AUTO_HANDLE_WIFI_PROBLEM_CANCEL */,
                detail: {
                    msg: '自动连接的wifi错误，用户取消自动连接',
                },
            });
            return false;
        }
        else {
            onProgress({
                code: "AUTO_HANDLE_WIFI_PROBLEM_SUCCESS" /* WifiConfStepCode.AUTO_HANDLE_WIFI_PROBLEM_SUCCESS */,
                detail: {
                    msg: '自动连接的wifi错误，用户设置自动连接',
                },
            });
            return true;
        }
    }
};
export const issueHandler = async ({ error, targetWifiInfo, wifiConfType, onProgress, softAPInfo, // 如果是softap的错误，可能是和热点错误有关
 }) => {
    let isHandled = false;
    switch (error.code) {
        case 'PROTOCOL_TIMEOUT':
        case 'CONNECT_SMARTCONFIG_FAIL':
            // softap先不处理
            if (wifiConfType !== 'SoftAp') {
                isHandled = await handleWifiIssue({ error, targetWifiInfo, onProgress });
            }
        case 'UDP_ERROR':
        case 'UDP_SEND_MSG_FAIL':
            // 可能是连接的网络不对，所以udp报错，比如4g情况下，发广播包的话，会直接出错
            if (wifiConfType === 'SoftAp') {
                if (softAPInfo) {
                    isHandled = await handleWifiIssue({ error, targetWifiInfo: softAPInfo, onProgress });
                }
            }
            else {
                isHandled = await handleWifiIssue({ error, targetWifiInfo, onProgress });
            }
            break;
        default:
            break;
    }
    return isHandled;
};
//# sourceMappingURL=issueHandler.js.map